package com.privas.application.services;

import java.util.*;

public class AttributesOptions {
    private List<String> attributesToPrivate;
    private List<String> attributesToHide;
    private List<String> attributesToShow;
    private List<String> idAttributes;
    private List<String> qidAttributes;
    private List<String> sensitiveAttributes;
    private Map<String, Attribute> attributes;

    public AttributesOptions() {
        attributesToPrivate = new ArrayList<>();
        attributesToHide = new ArrayList<>();
        attributesToShow = new ArrayList<>();
        idAttributes = new ArrayList<>();
        qidAttributes = new ArrayList<>();
        sensitiveAttributes = new ArrayList<>();
        attributes = new HashMap<>();
    }

    public void addAttribute(Attribute attribute) {
        this.attributes.put(attribute.getName(), attribute);

        if (attribute.isId()) {
            this.idAttributes.add(attribute.getName());
        }
        else if (attribute.isQId()) {
            this.qidAttributes.add(attribute.getName());
        }
        else if (attribute.isSA()) {
            this.sensitiveAttributes.add(attribute.getName());
        }
        else {
            this.attributesToShow.add(attribute.getName());
        }
    }

    public void addAttributeToPrivate(String attribute) {
        this.attributesToPrivate.add(attribute);
    }

    public List<String> getAttributesToPrivate() {
        return attributesToPrivate;
    }

    public void addAttributeToHide(String attribute) {
        this.attributesToHide.add(attribute);
    }

    public List<String> getAttributesToHide() {
        return attributesToHide;
    }

    public void addAttributeToShow(String attribute) {
        this.attributesToShow.add(attribute);
    }

    public List<String> getAttributesToShow() {
        return attributesToShow;
    }

    public void addAttributeId(String attribute) {
        this.idAttributes.add(attribute);
    }

    public List<String> getIdAttributes() {
        return idAttributes;
    }

    public void addAttributeQId(String attribute) {
        this.qidAttributes.add(attribute);
    }

    public List<String> getQidAttributes() {
        return qidAttributes;
    }

    public void addAttributeSensitive(String attribute) {
        this.sensitiveAttributes.add(attribute);
    }

    public List<String> getSensitiveAttributes() {
        return sensitiveAttributes;
    }

    public boolean isQIdAttribute(String attribute) {
        return this.qidAttributes.contains(attribute);
    }

    public boolean isSAAttribute(String attribute) {
        return this.sensitiveAttributes.contains(attribute);
    }

    public boolean hasSensitiveAttributes() {
        return this.sensitiveAttributes.size() > 0;
    }

    public Collection<Attribute> getAttributes() {
        return attributes.values();
    }

    public void calculateGeneralizations(Map<String, Generalization> generalizations) {
        for (Attribute attribute : attributes.values()) {
            attribute.calculateGeneralization(generalizations);
        }
    }
}
