package com.privas.presentation.api.controllers;

import com.privas.application.dto.ConnectionResult;
import com.privas.application.dto.PrivasLData;
import com.privas.application.dto.mapper.PrivasLMapper;
import com.privas.application.services.PrivasLGenerator;
import com.privas.data.grammar.PrivasLexer;
import com.privas.data.grammar.PrivasParser;
import com.privas.data.repositories.GenericRepository;
import com.privas.application.services.PrivacyDefinition;
import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

@RestController
@RequestMapping("/api")
public class HelloController {

    @GetMapping(path = "/hello")
    public ConnectionResult index(
            @RequestParam String url,
            @RequestParam String port,
            @RequestParam String database,
            @RequestParam String username,
            @RequestParam String password) {
//        String url = "localhost";
//        String port = "3306";
//        String database = "sakila";
//        String username = "root";
//        String password = "password";

        boolean success = GenericRepository.checkConnection(url, port, database, username, password);

        System.out.println(success);
        return new ConnectionResult(success);
    }

    @PostMapping(path = "/bye")
    public String index2(
            @RequestBody PrivasLData data) {

        System.out.println(data);
        return PrivasLGenerator.generate(data);
    }

    @CrossOrigin(origins = "http://localhost:4200")
    @PostMapping(path = "/bye2")
    public ResponseEntity<Resource> index3(
            @RequestBody PrivasLData data) {

        String content = PrivasLGenerator.generate(data);
        byte[] contentBytes = content.getBytes(StandardCharsets.UTF_8);

        HttpHeaders header = new HttpHeaders();
        header.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=repositoryDescription.privasL");
        header.add("Cache-Control", "no-cache, no-store, must-revalidate");
        header.add("Pragma", "no-cache");
        header.add("Expires", "0");

        InputStreamResource resource = new InputStreamResource(new ByteArrayInputStream(contentBytes));

        return ResponseEntity.ok()
                .headers(header)
                .contentLength(contentBytes.length)
                .contentType(MediaType.parseMediaType("application/octet-stream"))
                .body(resource);
    }

    @CrossOrigin(origins = "http://localhost:4200")
    @PostMapping(path = "/bye4")
    public ResponseEntity<PrivasLData> handleFileUpload(@RequestParam("file") MultipartFile file,
                                                        RedirectAttributes redirectAttributes) throws IOException {

        String content = new String(file.getBytes(), "UTF-8");
        redirectAttributes.addFlashAttribute("message",
                "You successfully uploaded " + file.getOriginalFilename() + "!");

        ANTLRInputStream antlrInputStream = new ANTLRInputStream(file.getInputStream());

        PrivasLexer lexer = new PrivasLexer(antlrInputStream);

        CommonTokenStream tokens = new CommonTokenStream(lexer);
        PrivasParser parser = new PrivasParser(tokens);

        PrivacyDefinition privacyDefinition = parser.privas().privacyDefinition;

        PrivasLData data = PrivasLMapper.map(privacyDefinition);

        return ResponseEntity.ok()
                .body(data);
    }
}
