import { Component, OnInit, Inject } from '@angular/core';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material';
import { faTrashAlt, faAsterisk, faExclamationTriangle, faSortDown } from '@fortawesome/free-solid-svg-icons';
import { faQuestionCircle } from '@fortawesome/free-regular-svg-icons';
import { EntityAttribute } from 'src/app/models/entity-attribute.model';
import { Entity } from 'src/app/models/entity.model';
import { GeneralizationService } from 'src/app/services/generalization.service';

@Component({
  selector: 'app-entity-dialog',
  templateUrl: './entity-dialog.component.html',
  styleUrls: ['./entity-dialog.component.scss']
})
export class EntityDialogComponent implements OnInit {
  faTrashAlt = faTrashAlt;
  faAsterisk = faAsterisk;
  faExclamationTriangle = faExclamationTriangle;
  faSortDown = faSortDown;
  faQuestionCircle = faQuestionCircle;

  public editMode = false;
  public attributes = [
    new EntityAttribute()
  ];

  public title = '';

  taxonomies = [];

  ppdpTypes = [
    {
      name: 'Id',
      value: 1,
      description: 'Identifier'
    },
    {
      name: 'QId',
      value: 2,
      description: 'Quasi-Identifier'
    },
    {
      name: 'SA',
      value: 3,
      description: 'Sensitive-Atribute'
    },
    {
      name: '',
      value: 0,
      description: 'Regular attribute'
    }
  ];

  constructor(
    private generalizationService: GeneralizationService,
    public dialogRef: MatDialogRef<EntityDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any) {
      if (data && data.entity) {
        this.title = data.entity.title;
        this.attributes = JSON.parse(JSON.stringify(data.entity.attributes));
        this.editMode = true;
      }
    }

  ngOnInit() {
    this.taxonomies = this.generalizationService.getGeneralizationNames();
  }

  addAttribute() {
    this.attributes.push(new EntityAttribute());
  }

  addEntity() {
    const newEntity = new Entity();

    newEntity.title = this.title;
    newEntity.attributes = this.attributes;

    this.dialogRef.close({
      added: true,
      entity: newEntity
    });
  }

  deleteAttribute(index) {
    this.attributes.splice(index, 1);
  }

  close() {
    this.dialogRef.close({added: false});
  }

}
