import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { MatDialog } from '@angular/material';
import { EntityDialogComponent } from '../../dialogs/entity-dialog/entity-dialog.component';
import { GeneralizationService } from 'src/app/services/generalization.service';
import { faSitemap } from '@fortawesome/free-solid-svg-icons';
import { Entity } from 'src/app/models/entity.model';

@Component({
  selector: 'app-define-entities',
  templateUrl: './define-entities.component.html',
  styleUrls: ['./define-entities.component.scss']
})
export class DefineEntitiesComponent implements OnInit {
  faSitemap = faSitemap;

  @Input()
  entities: Entity[];

  @Output()
  nextStep = new EventEmitter<boolean>();

  @Output()
  previousStep = new EventEmitter<boolean>();

  @Input()
  overviewMode: boolean;

  constructor(private dialog: MatDialog, private generalizationService: GeneralizationService) { }

  ngOnInit() {
  }

  onFinish() {
    this.nextStep.emit();
  }

  onPrevious() {
    this.previousStep.emit();
  }

  addEntity() {
    const dialogRef = this.dialog.open(EntityDialogComponent, {
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result.added === true) {
        this.entities.push(result.entity);
      }
      console.log(`Dialog result: ${JSON.stringify(result)}`);
    });
  }

  onEditEntity(entity: any, index: number) {
    const dialogRef = this.dialog.open(EntityDialogComponent, {
      data: {
        entity: this.entities[index]
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result.added === true) {
        this.entities[index] = result.entity;
      }
      console.log(`Dialog result: ${JSON.stringify(result)}`);
    });
  }

  onRemoveEntity(entity: any, index: number) {
    this.entities.splice(index, 1);
  }

  onShowGeneralizations() {
    this.generalizationService.toggle();
  }

}
