import { Component, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material';
import { EntityDialogComponent } from '../dialogs/entity-dialog/entity-dialog.component';
import { PrivacyModelsService } from 'src/app/services/privacy-models.service';
import { GeneralizationService } from 'src/app/services/generalization.service';
import { PrivacyLevel } from 'src/app/models/privacy-level.model';
import { PrivasLServiceService } from 'src/app/services/privas-l-service.service';
import { RepositoryData } from 'src/app/models/repository-data.model';
import { faCog } from '@fortawesome/free-solid-svg-icons';
import { saveAs } from 'file-saver';
import { DataService } from 'src/app/services/data.service';

@Component({
  selector: 'app-home',
  templateUrl: './home.component.html',
  styleUrls: ['./home.component.scss']
})
export class HomeComponent implements OnInit {
  faCog = faCog;
  activeStep = 0;

  entities = [
  ];

  privacyLevel: PrivacyLevel = new PrivacyLevel();

  repositoryData: RepositoryData = new RepositoryData();

  presentationMode = 'steps';

  generalizationData;

  constructor(private dialog: MatDialog,
              private privacyModelsService: PrivacyModelsService,
              private generalizationService: GeneralizationService,
              private privasLServiceService: PrivasLServiceService,
              private dataService: DataService) {
              }

  ngOnInit() {
    this.entities = this.dataService.entities;
    this.privacyLevel = this.dataService.privacyLevel;
    this.repositoryData = this.dataService.repositoryData;
    this.presentationMode = this.dataService.presentationMode;

    this.generalizationService.data$.subscribe(data => this.generalizationData = data);
  }

  addEntity() {
    const dialogRef = this.dialog.open(EntityDialogComponent, {
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result.added === true) {
        this.entities.push(result.entity);
      }
      console.log(`Dialog result: ${JSON.stringify(result)}`);
    });
  }

  onEditEntity(entity: any, index: number) {
    const dialogRef = this.dialog.open(EntityDialogComponent, {
      data: {
        entity: this.entities[index]
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result.added === true) {
        this.entities[index] = result.entity;
      }
      console.log(`Dialog result: ${JSON.stringify(result)}`);
    });
  }

  onRemoveEntity(entity: any, index: number) {
    this.entities.splice(index, 1);
  }

  setActiveStep(index) {
    this.activeStep = index;
  }

  onShowGeneralizations() {
    this.generalizationService.toggle();
  }

  onGenerate() {
    this.privasLServiceService.generateData(this.repositoryData,
      this.entities, this.privacyLevel, this.generalizationData).subscribe(data => {
        saveAs(data, 'repositoryDescription.privasL');
      },
        err => {
          alert('Problem while downloading the file.');
          console.error(err);
        });
  }

  onReview() {
    this.presentationMode = 'review';
  }

  downloadFile(data) {
    const blob = new Blob([data]);
    const url = window.URL.createObjectURL(blob);
    window.open(url);
  }
}
