import { Component, OnInit, ViewChild } from '@angular/core';
import { MatSidenav } from '@angular/material';
import { GeneralizationService } from 'src/app/services/generalization.service';
import { Tree } from 'src/app/models/tree.model';
import { TreeData } from 'src/app/models/tree-data.model';
import { faPencilAlt, faTrashAlt, faCheck, faLeaf } from '@fortawesome/free-solid-svg-icons';

@Component({
  selector: 'app-layout',
  templateUrl: './layout.component.html',
  styleUrls: ['./layout.component.scss']
})
export class LayoutComponent implements OnInit {
  faPencilAlt = faPencilAlt;
  faTrashAlt = faTrashAlt;
  faCheck = faCheck;
  faLeaf = faLeaf;

  @ViewChild('sidenav', null)
  public sidenav: MatSidenav;

  private trees: Tree[] = [];

  elementsIndicesBeingEdited = new Set<number>();

  constructor(private generalizationService: GeneralizationService) { }

  ngOnInit() {
    this.generalizationService.setSideNav(this.sidenav);

    this.generalizationService.data$.subscribe(data => {
      this.trees = data;
    });
  }

  onRemoveTree(index: number) {
    this.trees.splice(index, 1);
  }

  onRemoveTreeElement(tree: Tree, index: number) {
    tree.data.splice(index, 1);
  }

  onAddSubTree(tree: Tree) {
    tree.data.push(new TreeData(false, true));
  }

  onAddLeaf(tree: Tree) {
    tree.data.push(new TreeData(true, true));
  }

  onAddTree() {
    this.trees.push(new Tree());
    this.elementsIndicesBeingEdited.add(this.trees.length - 1);
  }

  isInEditMode(index: number) {
    return this.elementsIndicesBeingEdited.has(index);
  }

  toggleEdit(index: number) {
    if (this.isInEditMode(index)) {
      this.removeEditMode(index);
    } else {
      this.addEditMode(index);
    }
  }

  private addEditMode(index: number) {
    this.elementsIndicesBeingEdited.add(index);
  }

  private removeEditMode(index: number) {
    this.elementsIndicesBeingEdited.delete(index);
  }
}
