import { Component, OnInit, Input, ElementRef, AfterViewInit, EventEmitter, Output } from '@angular/core';
import { faPencilAlt, faTrashAlt, faCheck, faLeaf } from '@fortawesome/free-solid-svg-icons';
import { TreeData } from 'src/app/models/tree-data.model';

@Component({
  selector: 'app-tree',
  templateUrl: './tree.component.html',
  styleUrls: ['./tree.component.scss']
})
export class TreeComponent implements OnInit, AfterViewInit {
  faPencilAlt = faPencilAlt;
  faTrashAlt = faTrashAlt;
  faCheck = faCheck;
  faLeaf = faLeaf;

  @Input()
  data: TreeData[];

  @Input()
  collapsed = true;

  @Output() remove = new EventEmitter<number>();

  elementsIndicesBeingEdited = new Set<number>();

  constructor(private elementRef: ElementRef) { }

  ngOnInit() {
  }

  ngAfterViewInit() {
    this.addCollapsedClass();
  }

  toggle(event: Event, element: Element) {
    const targetElement: Element = event.target as Element;

    event.preventDefault();

    if (targetElement && targetElement.parentElement === element) {
      element.classList.toggle('collapsed');
    }
  }

  isOpen(element: Element) {
    return element.classList.contains('collapsed') === false;
  }

  isInEditMode(index: number) {
    return this.elementsIndicesBeingEdited.has(index);
  }

  toggleEdit(node: TreeData) {
    node.toggleEditMode();
  }

  onRemove(event: Event, index: number) {
    event.preventDefault();

    this.removeEditMode(index);
    this.remove.emit(index);
  }

  onRemoveChild(indexParent: number, indexChild: number) {
    this.data[indexParent].children.splice(indexChild, 1);
  }

  onAddElement() {
    this.data.push(new TreeData(false, true));
    this.elementsIndicesBeingEdited.add(this.data.length - 1);
  }

  onAddSubTree(node: TreeData) {
    node.children.push(new TreeData(false, true));
  }

  onAddLeaf(node: TreeData) {
    node.children.push(new TreeData(true, true));
  }

  private addEditMode(index: number) {
    this.elementsIndicesBeingEdited.add(index);
  }

  private removeEditMode(index: number) {
    this.elementsIndicesBeingEdited.delete(index);
  }

  private addCollapsedClass() {
    setTimeout(() => {
      const elements = this.elementRef.nativeElement.querySelectorAll('li');

      elements.forEach((element: Element) => {
        if (this.collapsed === false) {
          element.classList.remove('collapsed');
        }
      });
    }, 0);
  }
}
