import { Component, OnInit, EventEmitter } from '@angular/core';
import { faUndoAlt } from '@fortawesome/free-solid-svg-icons';
import { Router } from '@angular/router';
import { UploadOutput, UploadInput, UploadFile, humanizeBytes, UploaderOptions } from 'ngx-uploader';
import { PrivasLServiceService } from '../services/privas-l-service.service';
import { DataService } from '../services/data.service';
import { PrivacyDefinition } from '../models/privacy-definition.model';
import { GeneralizationService } from '../services/generalization.service';
import { PrivacyModelsService } from '../services/privacy-models.service';
import { Tree } from '../models/tree.model';

@Component({
  selector: 'app-landing',
  templateUrl: './landing.component.html',
  styleUrls: ['./landing.component.scss']
})
export class LandingComponent implements OnInit {
  faUndoAlt = faUndoAlt;

  options: UploaderOptions;
  files: UploadFile[];
  uploadInput: EventEmitter<UploadInput>;
  // tslint:disable-next-line:ban-types
  humanizeBytes: Function;
  dragOver: boolean;
  file: UploadFile;

  constructor(private router: Router,
              private privasLService: PrivasLServiceService,
              private dataService: DataService,
              private generalizationService: GeneralizationService,
              private privacyModelsService: PrivacyModelsService) {
    this.options = { concurrency: 1, maxUploads: 3 };
    this.files = []; // local uploading files array
    this.uploadInput = new EventEmitter<UploadInput>(); // input events, we use this to emit data to ngx-uploader
    this.humanizeBytes = humanizeBytes;
  }

  ngOnInit() {
  }

  onNew() {
    this.router.navigate(['/new']);
  }

  onLoadExisting() {
  }

  onUploadOutput(output: UploadOutput, i: boolean): void {
    if (output.type === 'addedToQueue' && typeof output.file !== 'undefined') { // add file to array when added
      this.file = output.file;
      this.privasLService.loadData(this.file.nativeFile).subscribe(response => {
        this.dataService.entities = response.entities;
        this.dataService.presentationMode = 'review';
        this.dataService.privacyLevel = PrivacyDefinition.deBuildPrivacyDefinition(response.privacyDefinition, this.privacyModelsService);
        this.dataService.repositoryData = response.repository;
        this.generalizationService.publishGeneralizations(Tree.mapToTrees(response.generalizationTrees));

        this.router.navigate(['/new']);
      }, error => {
        this.clearFile();
      });
    }
  }

  clearFile(): void {
    this.uploadInput.emit({ type: 'remove', id: this.file.id });
    this.file = null;
  }
}
