import { Injectable } from '@angular/core';
import { PrivacyLevel } from '../models/privacy-level.model';
import { PrivacyDefinition } from '../models/privacy-definition.model';
import { PrivasLData } from '../models/privas-ldata.model';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Tree } from '../models/tree.model';
import { RepositoryData } from '../models/repository-data.model';
import { Observable, throwError } from 'rxjs';
import { catchError, map, tap } from 'rxjs/operators';
import { PrivacyModelsService } from './privacy-models.service';

@Injectable({
  providedIn: 'root'
})
export class PrivasLServiceService {

  constructor(private http: HttpClient) { }

  generateData2(entities, privacyLevel: PrivacyLevel) {
    const privacyDefinition = PrivacyDefinition.buildPrivacyDefinition(privacyLevel.currentSelection);

    const privasLData = new PrivasLData();
    privasLData.entities = entities;
    privasLData.privacyDefinition = privacyDefinition;

    console.log(JSON.stringify(privasLData));
  }

  generateData(repositoryData: RepositoryData, entities, privacyLevel: PrivacyLevel, generalizationTrees: Tree[]) {
    const url = 'http://localhost:8080/api/bye2';

    const privacyDefinition = PrivacyDefinition.buildPrivacyDefinition(privacyLevel.currentSelection);

    const privasLData = new PrivasLData();
    privasLData.repository = repositoryData;
    privasLData.entities = entities;
    privasLData.privacyDefinition = privacyDefinition;
    privasLData.generalizationTrees = generalizationTrees;

    return this.http.post(url, privasLData, { responseType: 'blob' });
  }

  loadData(file: any): Observable<any> {
    const url = 'http://localhost:8080/api/bye4';

    const body = new FormData();
    body.append('file', file);

    return this.http.post<any>(url, body).pipe(
      map(response => response),
      tap(r => console.log(r)),
      catchError(this.handleError)
    );
  }

  private handleError(error: HttpErrorResponse) {
    if (error.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error.message);
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong,
      console.error(
        `Backend returned code ${error.status}, ` +
        `body was: ${error.error}`);
      console.log(error.error);
    }
    // return an observable with a user-facing error message
    return throwError(
      'Something bad happened; please try again later.');
  }
}
